/*
 * smdb.h
 *
 * Copyright 2002, 2004 by Anthony Howe.  All rights reserved.
 */

#ifndef __com_snert_lib_mail_smdb_h__
#define __com_snert_lib_mail_smdb_h__	1

#ifdef __cplusplus
extern "C" {
#endif

/***********************************************************************
 *** Constants
 ***********************************************************************/

#define SMDB_ACCESS_UNKNOWN	0
#define SMDB_ACCESS_OK		'O'	/* OK		O     */
#define SMDB_ACCESS_DISCARD	'D'	/* DISCARD	D     */
#define SMDB_ACCESS_ERROR	'E'	/* ERROR	E     */	
#define SMDB_ACCESS_FRIEND	'F'	/* FRIEND	F     */
#define SMDB_ACCESS_HATER	'H'	/* HATER	H     */
#define SMDB_ACCESS_RELAY	'L'	/* RELAY	  L   */
#define SMDB_ACCESS_REJECT	'J'	/* REJECT	  J   */
#define SMDB_ACCESS_SKIP	'S'	/* SKIP		S     */

/***********************************************************************
 *** Global Variables
 ***********************************************************************/

#ifdef LIBSNERT_WITH_BERKELEY_DB

#include <db.h>
#include <sys/stat.h>
#include <com/snert/lib/util/Vector.h>

#if DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 2 && DB_VERSION_PATCH >= 50
# define DBTXN		(DB_TXN *) 0,
#elif DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR == 1 && DB_VERSION_PATCH >= 25
# define DBTXN		(DB_TXN *) 0,
#elif DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR == 0
# define DBTXN
#elif DB_VERSION_MAJOR == 3 && DB_VERSION_MINOR >= 2
# define DBTXN
#elif defined(__bsdi__) || defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__)
# define DB_VERSION_MAJOR		1	/* Assume 1.85 */
# define DB_VERSION_MINOR		85
# define DB_NOTFOUND		1
# define db_strerror(x)		strerror(errno)
# include <fcntl.h>
#else
# error "Berkeley DB 1.85 or 3.2 or better required."
#endif

typedef struct smdb {
	DB *db;
	int dbtype;
	char *filename;
	struct stat finfo;
} smdb;

extern smdb *smdbAccess;
extern Vector smdbAliases;

/* 
 * NOTE these 3 functions are each a critical section and
 * need to be protected by a mutex in threaded applications.
 */
extern int smdbReload(smdb *sm);
extern int smdbReopen(smdb *sm);
extern smdb *smdbOpen(const char *filename, int dbtype);

extern int smdbGetValue(smdb *sm, DBT *key, DBT *value);
extern int smdbAliasesHasUser(Vector smdbAliasesList, char *user);

/*
 * Return one of SMDB_ACCESS_UNKNOWN, SMDB_ACCESS_OK, SMDB_ACCESS_REJECT, SMDB_ACCESS_DISCARD.
 */
extern int smdbAccessGetAccess(smdb *sm, DBT *key);

/*
 * Lookup
 *
 *	tag:a.b.c.d
 *	tag:a.b.c
 *	tag:a.b
 *	tag:a
 *
 * Return one of SMDB_ACCESS_UNKNOWN, SMDB_ACCESS_OK, SMDB_ACCESS_REJECT, SMDB_ACCESS_DISCARD.
 */
extern int smdbAccessGetIpAccess(smdb *sm, const char *tag, const char *ip);

/*
 * Lookup
 *
 *	tag:[ip]
 *	tag:[ipv6:ip]
 *	tag:some.sub.domain.tld
 *	tag:sub.domain.tld
 *	tag:domain.tld
 *	tag:tld
 *
 * Return one of SMDB_ACCESS_UNKNOWN, SMDB_ACCESS_OK, SMDB_ACCESS_REJECT, SMDB_ACCESS_DISCARD.
 */
extern int smdbAccessGetDomainAccess(smdb *sm, const char *tag, const char *domain);

/*
 * Lookup
 *
 *	tag:account@some.sub.domain.tld
 *	tag:some.sub.domain.tld
 *	tag:sub.domain.tld
 *	tag:domain.tld
 *	tag:tld
 *	tag:account@
 *
 * Return one of SMDB_ACCESS_UNKNOWN, SMDB_ACCESS_OK, SMDB_ACCESS_REJECT, SMDB_ACCESS_DISCARD.
 */
extern int smdbAccessGetEmailDomainUserAccess(smdb *sm, const char *tag, const char *email, const char *domain, const char *user);

/*
 * Lookup
 *
 *	tag:a.b.c.d
 *	tag:a.b.c
 *	tag:a.b
 *	tag:a
 *
 *	a.b.c.d
 *	a.b.c
 *	a.b
 *	a
 *
 * Return one of SMDB_ACCESS_UNKNOWN, SMDB_ACCESS_OK, SMDB_ACCESS_REJECT, SMDB_ACCESS_DISCARD.
 */
extern int smdbAccessIsIpOk(smdb *sm, const char *tag, char *ip);

/*
 * Lookup
 *
 *	tag:some.sub.domain.tld
 *	tag:sub.domain.tld
 *	tag:domain.tld
 *	tag:tld
 *
 *	some.sub.domain.tld
 *	sub.domain.tld
 *	domain.tld
 *	tld
 *
 * Return one of SMDB_ACCESS_UNKNOWN, SMDB_ACCESS_OK, SMDB_ACCESS_REJECT, SMDB_ACCESS_DISCARD.
 */
extern int smdbAccessIsDomainOk(smdb *sm, const char *tag, const char *host);

/*
 * Lookup
 *
 *	tag:account@some.sub.domain.tld
 *	tag:some.sub.domain.tld
 *	tag:sub.domain.tld
 *	tag:domain.tld
 *	tag:tld
 *	tag:account@
 *
 *	account@some.sub.domain.tld
 *	some.sub.domain.tld
 *	sub.domain.tld
 *	domain.tld
 *	tld
 *	account@
 *
 * Return one of SMDB_ACCESS_UNKNOWN, SMDB_ACCESS_OK, SMDB_ACCESS_REJECT, SMDB_ACCESS_DISCARD.
 */
extern int smdbAccessIsEmailDomainUserOk(smdb *sm, const char *tag, const char *email, const char *domain, const char *user);

#endif /* LIBSNERT_WITH_BERKELEY_DB */

#define SMDB_DEBUG_ALL		(~0)
#define SMDB_DEBUG_INFO		1
#define SMDB_DEBUG_GET		2
#define SMDB_DEBUG_SET		4

extern void smdbSetDebugMask(int mask);

/*
 * Parse the sendmail.cf and initialise global variables.
 *
 *	smdbAccess		(only with Berkeley DB support)
 *	smdbAliases		(only with Berkeley DB support)
 * 	smMasqueradeAs
 *	smClientPortInet4
 *	smClientPortInet6
 * 
 * @param cf
 *	The sendmail.cf file path.
 *
 * @param flags
 *	A bit mask of flags used to control which sendmail databases
 *	should be opened.
 *
 * @return
 *	Zero (0) for success, otherwise -1 on error.
 */
extern int readSendmailCf(char *cf, long flags);

#define SMDB_OPEN_ALL		(SMDB_OPEN_ACCESS|SMDB_OPEN_ALIASES)
#define SMDB_OPEN_ACCESS	1
#define SMDB_OPEN_ALIASES	2

extern char *smMasqueradeAs;

typedef struct {
	long port;
	int family;
	char *address;
	int useForHelo;
	int dontUseAuth;
	int dontUseStartTls;
} SmClientPortOptions;

SmClientPortOptions smClientPortInet4;
SmClientPortOptions smClientPortInet6;

#ifdef  __cplusplus
}
#endif

#endif /* __com_snert_lib_mail_smdb_h__ */

