/*
 * Text.h
 *
 * Copyright 2001, 2004 by Anthony Howe.  All rights reserved.
 */

#ifndef __com_snert_lib_util_Text_h__
#define __com_snert_lib_util_Text_h__	1

#ifndef __com_snert_lib_util_Vector_h__
#include <com/snert/lib/util/Vector.h>
#endif

#ifndef __com_snert_lib_util_Buf_h__
#include <com/snert/lib/util/Buf.h>
#endif

#ifndef BUFSIZ
#include <stdio.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

#ifndef CompareFunctionType
#define CompareFunctionType		1
typedef int (*CompareFunction)(const void *, const void *);
#endif

#ifndef LongCompareFunctionType
#define LongCompareFunctionType		1
typedef long (*LongCompareFunction)(const void *, const void *);
#endif

extern char *TextDup(const char *);
extern char *TextSubstring(const char *, long offset, long length);
extern long TextHash(long, const char *);
extern int TextIsEmpty(const char *);
extern int TextIsInteger(const char *, int);
extern int TextBackslash(char);
extern char *TextToken(const char *, const char **, const char *, int);
extern Vector TextSplit(const char *, const char *, int);
extern char *TextJoin(const char *, Vector);

extern Buf TextExpand(Buf s, long col);
extern char *TextHexEncode(Buf b);

extern long TextCopy(char *t, long tlength, char *s, long slength);
extern long TextCopyN(char *target, char *source, long length);

/*
 * Comparision functions; return -ve, 0, +ve for <, ==, >.
 */
extern int TextSensitiveCompare(const void *xp, void const *yp);
extern int TextSensitiveCompareN(const void *xp, void const *yp, long len);
extern int TextInsensitiveCompare(const void *xp, void const *yp);
extern int TextInsensitiveCompareN(const void *xp, void const *yp, long len);

/*
 * Comparision functions; return -1 no match, 0 <= length
 */
extern long TextSensitiveStartsWith(const char *text, const char *prefix);
extern long TextInsensitiveStartsWith(const char *text, const char *prefix);
extern long TextSensitiveEndsWith(const char *text, const char *suffix);
extern long TextInsensitiveEndsWith(const char *text, const char *suffix);

/*
 * Line input routines.
 */
extern int TextInputLine(FILE *fp, char *line, int size);
extern int TextReadLine(int fd, char *line, int size);

/*
 * Search routines.
 */
extern int TextCountOccurences(const char *str, const char *sub);
extern long TextSensitiveFind(const char *text, const char *sub);
extern long TextInsensitiveFind(const char *text, const char *sub);

/*
 * In-place alterations.
 */
extern void TextInvert(char *str, long offset, long length);
extern void TextLower(char *str, long offset, long length);
extern void TextUpper(char *str, long offset, long length);
extern void TextReverse(char *str, long offset, long length);

extern void boundOffsetLength(long slength, long *offset, long *length);

#ifdef  __cplusplus
}
#endif

#endif /* __com_snert_lib_util_Text_h__ */
