/*
 * Vector.h
 *
 * Dynamic Vector API
 *
 * Copyright 2001, 2004 by Anthony Howe.  All rights reserved.
 */

#ifndef __com_snert_lib_util_Vector_h__
#define __com_snert_lib_util_Vector_h__	1

#ifdef __cplusplus
extern "C" {
#endif

#ifndef CompareFunctionType
#define CompareFunctionType	1
typedef int (*CompareFunction)(const void *a, const void *b);
#endif

#ifndef WalkFunctionType
#define WalkFunctionType	1
typedef int (*WalkFunction)(void *object, void * data);
#endif

#ifndef WALK_OK
#define	WALK_OK			0
#define WALK_STOP		1
#define WALK_REMOVE_ENTRY	2
#endif

typedef struct Vector {
	void (*destroy)(struct Vector *);	

	void **(*getBase)(struct Vector *);
	long (*getLength)(struct Vector *);
	long (*getCapacity)(struct Vector *);

	int (*add)(struct Vector *, void *object);
	int (*set)(struct Vector *, long index, void *object);
	int (*get)(struct Vector *, long index, void **object);
	int (*walk)(struct Vector *, WalkFunction, void *data);
	int (*remove)(struct Vector *, long index, long length);
	
	/*
	 * In-place modifications
	 */
	int (*reverse)(struct Vector *, long a, long b);
	int (*move)(struct Vector *, long a, long b, long c);
	int (*sort)(struct Vector *, CompareFunction);
} *Vector;

extern Vector VectorCreate(long capacity);

/*
 * Compatibility with old API.
 */
#define VectorDestroy(v)		(v)->destroy(v)
#define VectorBase(v)			(v)->getBase(v)
#define VectorLength(v)			(v)->getLength(v)
#define VectorCapacity(v)		(v)->getCapacity(v)
#define VectorAdd(v, o)			(v)->add(v, o)
#define VectorGet(v, i)			(v)->get(v, i)
#define VectorSet(v, i, o)		(v)->set(v, i, o)
#define VectorRemove(v, i, l)		(v)->remove(v, i, l)
#define VectorReverse(v, a, b)		(v)->reverse(v, a, b)
#define VectorMove(v, a, b, c)		(v)->move(v, a, b, c)
#define VectorSort(v, c)		(v)->sort(v, c)
#define VectorWalk(v, f, d)		(v)->sort(v, f, d)

#ifdef  __cplusplus
}

#endif

#endif /* __com_snert_lib_util_Vector_h__ */

