/*
 * Vector.h
 *
 * Dynamic Vector API
 *
 * Copyright 2001, 2004 by Anthony Howe.  All rights reserved.
 */

#ifndef __com_snert_lib_util_Vector_h__
#define __com_snert_lib_util_Vector_h__	1

#ifdef __cplusplus
extern "C" {
#endif

#ifndef CompareFunctionType
#define CompareFunctionType	1
typedef int (*CompareFunction)(const void *, const void *);
#endif

#ifndef WALK_OK
#define	WALK_OK			0
#define WALK_STOP		1
#define WALK_REMOVE_ENTRY	2
#endif

typedef int (*VectorWalkFunction)(void *entry, void *data);

typedef void *Vector;

/*
 * Create a vector with an initial capacity and the entry
 * destructor (onRemove) function. Note that a vector supports
 * only entries of one type at a time.
 */
extern Vector VectorCreate(long capacity, void (*destroy)(void *entry));

/*
 * Destroy all the vector's entries and the vector itself.
 */
extern void VectorDestroy(Vector);

/*
 * Access fields.
 */
extern long VectorCapacity(Vector);
extern long VectorLength(Vector);
extern void **VectorBase(Vector);

/*
 * Append. Return 0 for success or -1 on error.
 */
extern int VectorAdd(Vector, void *object);

/*
 * Extract value.
 */
extern void *VectorGet(Vector, long index);

/*
 * Replace value.
 */
extern void VectorSet(Vector, long index, void *object);

/*
 * In-place modifications
 */
extern void VectorSort(Vector, CompareFunction);
extern void VectorRemove(Vector, long offset, long length);
extern void VectorReverse(Vector, long start, long finish);

/*
 * Move. Return 0 for success or -1 on error.
 */
extern int VectorMove(Vector, long start, long finish, long to);

/*
 * Walk through every entry in the vector, applying the supplied
 * function to every entry.
 */
extern void VectorWalk(Vector, VectorWalkFunction, void *data);

/*
 * Predefined VectorWalkFunction
 */
extern int VectorWalkIgnore(void *object, void *data);

/*
 * Set the object remove function. The defaut is VectorRemoveAbort().
 */
extern void VectorOnRemove(Vector, void (*destroy)(void *entry));

/*
 * Predefined VectorRemoveFunction
 */
extern void VectorRemoveAbort(void *object);
extern void VectorRemoveIgnore(void *object);

#ifdef  __cplusplus
}

#endif

#endif /* __com_snert_lib_util_Vector_h__ */

